/*
 * This code is based on the Foedus IMAP server, originally written by David Weitzman.  
 * http://sourceforge.net/projects/foedus
 */
package rice.email.proxy.pop3;

import rice.email.proxy.pop3.commands.*;
import rice.email.proxy.user.*;
import rice.environment.Environment;
import rice.environment.logging.Logger;

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;

public class Pop3Handler {
  
  Pop3CommandRegistry _registry;
  Pop3Connection _conn;
  UserManager _manager;
  Pop3State _state;
  boolean _quitting;
  String _currentLine;
  Environment environment;
  InetAddress localHost;
  Logger logger;
  
  public Pop3Handler(InetAddress localHost, Pop3CommandRegistry registry, UserManager manager, Environment env) {
    this.localHost = localHost;
    _registry = registry;
    _manager = manager;
    environment = env;
    logger = environment.getLogManager().getLogger(Pop3Handler.class, null);
  }
  
  public InetAddress getLocalHost() {
    return localHost;
  }
  
  public void handleConnection(Socket socket) throws IOException {
    try {
      _conn = new Pop3Connection(this, socket, environment);
      _state = new Pop3State(_manager);
      _quitting = false;
      
      sendGreetings();
      
      while (!_quitting) {
        handleCommand();
      }
      
      _conn.close();
    } catch (SocketTimeoutException ste) {
      _conn.println("421 Service shutting down and closing transmission channel");
    }
    catch (Exception e) {
      if (logger.level <= Logger.WARNING) logger.logException("", e);
    } finally {
      try {
        socket.close();
      } catch (IOException ioe) {
        if (logger.level <= Logger.WARNING) logger.logException("", ioe);
      }
    }
  }
  
  void sendGreetings() {          
    String challenge = "<" + environment.getTimeSource().currentTimeMillis() + "@localhost>";
//    try {
      challenge = "<" + environment.getTimeSource().currentTimeMillis() + "@" + getLocalHost().getHostName() + ">";
//    } catch (UnknownHostException e) {
//    } 

    _state.setChallenge(challenge);
    
    _conn.println("+OK POP3 server ready " + challenge);
  }
  
  void handleCommand() throws IOException {
    _currentLine = _conn.readLine();
    
    if (_currentLine == null) {
      quit();
      
      return;
    }
    
    String commandName = new StringTokenizer(_currentLine, " ").nextToken().toUpperCase();
    Pop3Command command = _registry.getCommand(commandName);
    
    if (command == null) {
      _conn.println("-ERR Command not recognized");
    } else if (! command.isValidForState(_state)) {
      _conn.println("-ERR Command not valid for this state");
    } else {
      command.execute(_conn, _state, _currentLine);
    }
  }
  
  public void quit() {
    _quitting = true;
  }
}
